/*!\class CamacIO
\author Alexandre A. P. Suaide 

<p>
This class is derived from PelIO and overwrites some of PelIO's methods to
allow connection to a CAMAC system for data acquisition. 
*/
#ifndef CamacIO_HH
#define CamacIO_HH

#include "PelIO.h"
#include "TROOT.h"

class CamacIO : public PelIO
{
  private:    
    bool               mIsCamac;  ///< bool variable to flag if CAMAC is being used
    int                readBlock(); ///< Read event block from CAMAC
      
  public:
                       CamacIO(char* name="CamacIO",char *title="Pellectron IO"); ///< Default constructor
    virtual            ~CamacIO();///< Default destructor
    
    // Input/output methods
    int                openInput(char* file="/dev/c111a0"); ///< Open input stream
    int                closeInput(); ///< Close input stream
    
    // CAMAC related methods
    int                loadEVO(char*); ///< Load Event Handler file into memory
    int                startCamac(); ///< Start CAMAC acquisition
    int                stopCamac(); ///< Stop CAMAC acquisition
    int                getNEventsLost(); ///< Get number of events lost during acquisition
    int                getNUsedBuffers(); ///< Get number of used buffers
    int                getNBuffers(); ///< Get number of buffers
    int                readScalers(int,int*,int=0); ///< Read scalers
    int                clearScalers(int,int=0); ///< Clear scalers
    int                setCamacBufferSize(int); ///<Sets CAMAC buffer size in byets
        
  ClassDef(CamacIO,1)
};
#endif
