#include "defines.h"
#include "TROOT.h"
TString baseName;

void reverse(char*);

int main(int argc, char** argv)
{
  if(argc==1)
  {
    cout <<"usage: adac+ filename\n";
    return 0;
  }
  baseName = argv[1];
  if(baseName.EndsWith(".evp")) baseName = baseName(0,baseName.Length()-4);
  reverse((char*)baseName.Data());
}

void reverse(char* fileName)
{
  char line[400];
  int KSYS=0,KAUX=0,NAUX=0,NSHD=0,ADINIT=0;
  int NC = 0;
  TString RN[4];
  TString OP[10];
  RN[1] = "PAT";
  RN[2] = "EX";
  RN[3] = "CA";
  
  OP[0] = "ANY";
  OP[1] = "NONE";
  OP[2] = "GT";
  OP[3] = "LT";
  OP[4] = "GE";
  OP[5] = "LE";
  OP[6] = "NE";
  OP[7] = "EQ";
  
  unsigned int C   = 0;
  unsigned int CMD = 0;
  unsigned int ARG = 0;
  unsigned int REG1 = 0;
  unsigned int REG2 = 0;
  unsigned int R1 = 0;
  unsigned int R2 = 0;
  unsigned int N = 0;
  unsigned int A = 0;
  unsigned int F = 0;
  unsigned int O = 0;
  
  TString file=fileName;
  file+=".evp";
  FILE *in = fopen((char*)file.Data(),"r");
  if(!in)
  {
    cout <<"File does not exist or problem reading file\n";
    exit(0);
  }
  
  // reading header
  fgets(line,400,in); 
  cout <<"TITLE = "<<line<<endl;
  
  fscanf(in,"%d %d %d %d %d %d",&NC,&KSYS,&KAUX,&NAUX,&NSHD,&ADINIT);
  cout <<1<<endl;
  cout<<"NC = "<<NC<<"  KSYS = "<<KSYS<<"  KAUX = "<<KAUX<<"  NAUX = "<<NAUX<<" 	NSHD = "<<NSHD<<"  ADINIT = "<<ADINIT<<endl;  
  
  for(int i=0;i<NC;i++)
  {
    fscanf(in,"%x  ",&C);
    sprintf(line," %4d | %08x | ",i+1,C); cout <<line;
    
    if(C<=0x007FFFFF) // class 0 
    {
      if(C==0x00000000)      cout <<" NOP \n";
      else if(C==0x00000001) cout <<" ABORT \n";
      else if(C==0x00100000) cout <<" RTRN \n";
      else if(C==0x00100001) cout <<" END \n";
      else if(C==0x00100002) cout <<" INIT \n";
      else if(C==0x00100003) cout <<" BRUR \n";
      else
      {
        CMD = C & 0xFFFF0000;
        ARG = C & 0x0000FFFF;        
        if(CMD == 0x00400000)      
				{
				  if(i+1>ADINIT) ARG+=ADINIT;
					cout <<" SPB "<<ARG<<endl;
				}
        else if(CMD == 0x00500000) 
				{
				  if(i+1>ADINIT) ARG+=ADINIT;
					cout <<" BRU "<<ARG<<endl;
        }
				else if(CMD == 0x00600000) cout <<" SETC "<<ARG<<endl;
        else if(CMD == 0x00700000) cout <<" DLAY "<<ARG<<endl;
        else  cout <<"COMMAND NOT FOUND "<<endl;
      }
    }
    else if(C<=0x00FFFFFF) // class 1
    {
      CMD = C & 0x00F00000;
      if(CMD == 0x00C00000) // FLUSH
      {
        N   = (C & 0x000F0000)>>16;
        ARG = (C & 0x0000FFFF);
        cout <<" FLUSH "<<N<<","<<ARG<<endl;        
      }
      else if(CMD == 0x00F00000) // out or par
      {
        REG1= (C & 0x000F0000)>>16;
        ARG = (C & 0x0000FFFF);
        if(REG1>=0x8) // PAR
        {
          REG1 = REG1 & 0x3;
          if(REG1==0) cout <<" PAR "<<hex<<ARG<<"H"<<dec<<endl;
          else cout <<" PAR "<<RN[REG1].Data()<<ARG<<endl;          
        }
        else
        {
          if(REG1==0) cout <<" OUT "<<hex<<ARG<<"H"<<dec<<endl;
          else cout <<" OUT "<<RN[REG1].Data()<<ARG<<endl;
        }
      }
      else // NAF
      {
        REG1 = (C & 0x00300000)>>20;
        R1   = (C & 0x000F0000)>>16;
        if(C & 0x00008000) // NAFX
        {
          N    = (C & 0x0f00);
          A    = (C & 0x00f0)>>4;
          F    = (C & 0x000f);        
          cout <<" NAFX "<<RN[R_EXX].Data()<<N<<","<<RN[R_EXX].Data()<<A<<","<<RN[R_EXX].Data()<<F<<" "<<RN[REG1].Data()<<R1<<endl;
        }
        else // NAF
        {
          N    = (C & 0x001f);
          F    = (C & 0x03e0)>>5;
          A    = (C & 0x3c00)>>10;
          cout <<" NAF "<<N<<","<<A<<","<<F<<" "<<RN[REG1].Data()<<R1<<endl;
        }
      }
    }
    else if(C<=0x7FFFFFFF) // class 2
    {
      CMD = C & 0x0F000000;
      REG1 = (C & 0x00C00000)>>22;
      REG2 = (C & 0x00300000)>>20;
      R2   = (C & 0x000F0000)>>16;  
      ARG  = (C & 0x0000FFFF); // OR R1
      if(CMD==0x01000000) // MERG A,B
      {
        if(REG1==0) cout <<" MERG "<<hex<<ARG<<dec<<"H,"<<RN[REG2].Data()<<R2<<endl;
        else cout <<" MERG "<<RN[REG1].Data()<<ARG<<","<<RN[REG2].Data()<<R2<<endl;
      }
      else if(CMD==0x02000000) // MOV A,B
      {
        if(REG1==0) cout <<" MOV "<<hex<<ARG<<dec<<"H,"<<RN[REG2].Data()<<R2<<endl;
        else cout <<" MOV "<<RN[REG1].Data()<<ARG<<","<<RN[REG2].Data()<<R2<<endl;
      }
      else if(CMD==0x03000000) // INC
      {
        if(REG1==0) cout <<" INC "<<RN[REG2].Data()<<R2<<","<<hex<<ARG<<dec<<"H"<<endl;
        else cout <<" INC "<<RN[REG2].Data()<<R2<<","<<RN[REG1].Data()<<ARG<<endl;
      }
      else if(CMD==0x04000000) // DEC
      {
        if(REG1==0) cout <<" DEC "<<RN[REG2].Data()<<R2<<","<<hex<<ARG<<dec<<"H"<<endl;
        else cout <<" DEC "<<RN[REG2].Data()<<R2<<","<<RN[REG1].Data()<<ARG<<endl;
      }
      else if(CMD==0x05000000) // DEC
      {
        if(REG1==0) cout <<" AND "<<RN[REG2].Data()<<R2<<","<<hex<<ARG<<dec<<"H"<<endl;
        else cout <<" AND "<<RN[REG2].Data()<<R2<<","<<RN[REG1].Data()<<ARG<<endl;
      }
      else if(CMD==0x06000000) // DEC
      {
        if(REG1==0) cout <<" OR "<<RN[REG2].Data()<<R2<<","<<hex<<ARG<<dec<<"H"<<endl;
        else cout <<" OR "<<RN[REG2].Data()<<R2<<","<<RN[REG1].Data()<<ARG<<endl;
      }
      else if(CMD==0x07000000) // DEC
      {
        cout <<" NOT "<<RN[REG2].Data()<<R2<<endl;
      }
      else if(CMD==0x08000000) // DEC
      {
        if(REG1==0) cout <<" RSH "<<RN[REG2].Data()<<R2<<","<<hex<<ARG<<dec<<"H"<<endl;
        else cout <<" RSH "<<RN[REG2].Data()<<R2<<","<<RN[REG1].Data()<<ARG<<endl;
      }
      else if(CMD==0x09000000) // DEC
      {
        if(REG1==0) cout <<" LSH "<<RN[REG2].Data()<<R2<<","<<hex<<ARG<<dec<<"H"<<endl;
        else cout <<" LSH "<<RN[REG2].Data()<<R2<<","<<RN[REG1].Data()<<ARG<<endl;
      }
      else if(CMD==0x0A000000) // DEC
      {
        if(REG1==0) cout <<" MOD "<<RN[REG2].Data()<<R2<<","<<hex<<ARG<<dec<<"H"<<endl;
        else cout <<" MOD "<<RN[REG2].Data()<<R2<<","<<RN[REG1].Data()<<ARG<<endl;
      }
      else if(CMD==0x0B000000) // DEC
      {
        if(REG1==0) cout <<" MUL "<<RN[REG2].Data()<<R2<<","<<hex<<ARG<<dec<<"H"<<endl;
        else cout <<" MUL "<<RN[REG2].Data()<<R2<<","<<RN[REG1].Data()<<ARG<<endl;
      }
      else if(CMD==0x0C000000) // DEC
      {
        if(REG1==0) cout <<" DIV "<<RN[REG2].Data()<<R2<<","<<hex<<ARG<<dec<<"H"<<endl;
        else cout <<" DIV "<<RN[REG2].Data()<<R2<<","<<RN[REG1].Data()<<ARG<<endl;
      }
      else  cout <<"COMMAND NOT FOUND "<<endl;
    }
    else // class 3
    {
      O    = (C & 0x70000000)>>28;
      REG1 = (C & 0x0C000000)>>26;
      REG2 = (C & 0x03000000)>>24;
      R1   = (C & 0x00F00000)>>20;  
      R2   = (C & 0x000F0000)>>16;  
      ARG  = (C & 0x0000FFFF); 
      cout <<" SKIP ";
      if(REG1>0) cout <<RN[REG1].Data()<<R1;
      else cout <<hex<<ARG<<"H"<<dec;
      cout<<"."<<OP[O].Data()<<".";
      if(REG2>0) cout <<RN[REG2].Data()<<R2;
      else cout <<hex<<ARG<<"H"<<dec;
      cout <<endl;
    }
  }

}

